@@ -7,10 +7,10 @@
 */
 
 /* .Fortran calls */
-extern void F77_NAME(gdist)(void *, void *, void *, void *, void *, void *, void *, void *);
+extern void F77_NAME(gdist)(void *, void *, void *, void *, void *, void *, void *, void *, void *);
 
 static const R_FortranMethodDef FortranEntries[] = {
-    {"gdist", (DL_FUNC) &F77_NAME(gdist), 8},
+    {"gdist", (DL_FUNC) &F77_NAME(gdist), 9},
     {NULL, NULL, 0}
 };
 
